@@ -5,6 +5,7 @@ import numbers
 from collections import OrderedDict
 from decimal import Decimal
 from importlib import import_module
+from django.utils.html import escape
 
 import xlsxwriter
 from django.db.models import BinaryField, ForeignKey
@@ -94,7 +95,7 @@ def _get_table_fields(Model, for_export=False):
                 '_current_app_name',
                 '_round_number',
                 '_current_page_name',
-                'status',
+                '_monitor_note',
                 '_last_page_timestamp',
             ]
 
@@ -106,10 +107,7 @@ def _get_table_fields(Model, for_export=False):
             and f not in ['id', 'group_id', 'subsession_id']
         ]
 
-        if for_export:
-            return ['id_in_group'] + subclass_fields + ['payoff']
-        else:
-            return ['id_in_group', 'role'] + subclass_fields + ['payoff']
+        return ['id_in_group', 'role'] + subclass_fields + ['payoff']
 
     if issubclass(Model, BaseGroup):
         subclass_fields = [
@@ -152,10 +150,10 @@ def sanitize_for_csv(value) -> str:
 def sanitize_for_live_update(value):
     # force_text is necessary e.g. for CountryField, which is otherwise
     # not Json serializable
-    value = str(sanitize_for_csv(value))
+    value = escape(sanitize_for_csv(value))
     MAX_LENGTH = 30
     if len(value) > MAX_LENGTH:
-        return value[:MAX_LENGTH] + '...'
+        return value[:MAX_LENGTH] + '…'
     return value
 
 
@@ -312,8 +310,9 @@ def get_rows_for_wide_csv_round(app_name, round_number, sessions):
             subsession_rows = []
 
             for player in players:
-                # because player.payoff is a property
+                # because these are properties
                 player['payoff'] = player['_payoff']
+                player['role'] = player['_role']
                 row = []
                 all_objects = {
                     'player': player,
@@ -373,8 +372,9 @@ def get_rows_for_csv(app_name):
     ]
 
     for player in players:
-        # because player.payoff is a property
+        # because these are properties
         player['payoff'] = player['_payoff']
+        player['role'] = player['_role']
         row = []
         all_objects = {'player': player}
         for model_name in value_dicts:
@@ -390,6 +390,22 @@ def get_rows_for_csv(app_name):
     return rows
 
 
+def get_rows_for_monitor(participants) -> list:
+    field_names = get_field_names_for_live_update(Participant)
+    callable_fields = {'_numeric_label', '_current_page'}
+    rows = []
+    for participant in participants:
+        row = {}
+        for field_name in field_names:
+            value = getattr(participant, field_name)
+            if field_name in callable_fields:
+                value = value()
+            row[field_name] = value
+        row['id_in_session'] = participant.id_in_session
+        rows.append(row)
+    return rows
+
+
 def get_rows_for_live_update(subsession: BaseSubsession):
 
     models_module = import_module(subsession.__module__)
