@@ -1,11 +1,12 @@
-from otree.common import add_field_tracker, in_round, in_rounds
+from otree.common import in_round, in_rounds
 
 from otree.db import models
 from otree.models.fieldchecks import ensure_field
 from django.db import models as djmodels
+from otree.db.idmap import PlayerIDMapMixin
 
 
-class BasePlayer(models.OTreeModel):
+class BasePlayer(models.OTreeModel, PlayerIDMapMixin):
     """
     Base class for all players.
     """
@@ -43,6 +44,13 @@ class BasePlayer(models.OTreeModel):
 
     round_number = models.PositiveIntegerField(db_index=True)
 
+    _role = models.StringField()
+
+    # as a property, that means it's overridable
+    @property
+    def role(self):
+        return self._role
+
     @property
     def payoff(self):
         return self._payoff
@@ -71,10 +79,6 @@ class BasePlayer(models.OTreeModel):
             fmt_string = '<Player {}>'
         return fmt_string.format(id_in_subsession)
 
-    def role(self):
-        # you can make this depend of self.id_in_group
-        return ''
-
     def in_round(self, round_number):
         return in_round(type(self), round_number, participant=self.participant)
 
@@ -114,7 +118,5 @@ class BasePlayer(models.OTreeModel):
         )
         ensure_field(cls, 'group', group_field)
 
-        add_field_tracker(cls)
-
     def start(self):
         pass
